/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is cookiebutton.
 *
 * The Initial Developer of the Original Code is
 * Pang Lih Wuei (basic@mozdev.org).
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
window.addEventListener("load",function (e) {
	var broadcaster = document.getElementById("cookie-perm-broadcaster");
	var browser = document.getElementById("content");
	broadcaster._cookiePermission = {
		debug : function () {
		},
		// constant from nsCookiePermission.cpp
		ALLOW_SESSION : 8,
		// preference service
		prefService : Components.classes["@mozilla.org/preferences;1"].getService()
								.QueryInterface(Components.interfaces.nsIPrefService),
		
		// permission manager
		permissionManager : Components.classes["@mozilla.org/permissionmanager;1"].getService()
											.QueryInterface(Components.interfaces.nsIPermissionManager),
		
		// reference to tab browser
		browser :  browser,
		
		// reference to broadcaster that this object is tied to
		broadcaster : broadcaster,
				// init function
		init : function () {
		
			// setup cookie pref branch reference
			if(!this.prefBranch) 
				this.prefBranch = this.prefService.getBranch("network.cookie.")
												.QueryInterface(Components.interfaces.nsIPrefBranch)
												.QueryInterface(Components.interfaces.nsIPrefBranchInternal);

			// setup observer on cookieBehavior,warnAboutCookies preferences
			if(!this.prefObserver) {
				this.prefObserver = {
					_broadcaster : this,
					observe : function ( subject , topic , data ) {
						this._broadcaster.update();
					},
					QueryInterface : function (aIID) {
						if (aIID.equals(Components.interfaces.nsIObserver) ||
							aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
							aIID.equals(Components.interfaces.nsISupports))
								return this;
						throw Components.results.NS_NOINTERFACE;
					}
				};
				try {
					this.prefBranch.addObserver("cookieBehavior", this.prefObserver,true);
				}
				catch (e) {dump("could not observe network.cookie.cookieBehavior"); }
				try {
					this.prefBranch.addObserver("warnAboutCookies", this.prefObserver,true);
				}
				catch (e) {dump("could not observe network.cookie.warnAboutCookies"); }
			}
			// setup browser location webProgressListener
			if(!this.locationListener) {
				this.locationListener = {
					_broadcaster : this,
					onLocationChange : function ( webProgress , request , location ) {
						this._broadcaster.update();
					},
					// to avoid error messages
					onProgressChange : function (webProgress,request,curSelfProgress,maxSelfProgress,curTotalProgress,maxTotalProgress) {
						// shouldn't need this here, but we seem to be hitting a firebird bug where  onLocationChange sometimes fail to trigger
						this._broadcaster.update();
					},
					onSecurityChange : function (webProgress,request,state) {
						// shouldn't need this here, but we seem to be hitting a firebird bug where  onLocationChange sometimes fail to trigger
						this._broadcaster.update();
					},
					onStateChange : function (webProgress,request,stateFlags,status) {
						// shouldn't need this here, but we seem to be hitting a firebird bug where  onLocationChange sometimes fail to trigger
						this._broadcaster.update();
					},
					onStatusChange : function (webProgress,request,status,message) {
						// shouldn't need this here, but we seem to be hitting a firebird bug where  onLocationChange sometimes fail to trigger
						this._broadcaster.update();
					},
					onLinkIconAvailable : function (href) {
						// shouldn't need this here, but we seem to be hitting a firebird bug where onLocationChange sometimes fail to trigger
						this._broadcaster.update();
					}
				};
				this.browser.addProgressListener(this.locationListener,Components.interfaces.nsIWebProgress.NOTIFY_LOCATION);
			}
		},

		// update button
		update : function () {
			var location = this.browser.currentURI;
			// check if we are http or https, if not disable button
			if(!location || (location.scheme.search(/^https?$/) == -1)) {
				this.broadcaster.removeAttribute("value");
				this.broadcaster.setAttribute("label",this.labelGenerator("default"));
				this.broadcaster.setAttribute("disabled","true");
				return false;
			}
			
			var value = this.getPermission(location);
			
			// set button look
			this.broadcaster.setAttribute("value",value);
			this.broadcaster.setAttribute("label",this.labelGenerator(value,location));
			this.broadcaster.removeAttribute("disabled");
			return true;
		},

		// get default premission from preferences
		getDefaultPref: function () {
			var behavior = 0;

			// for some earlier firebird builds
			try {
				if(!this.prefBranch.getBoolPref("enable")) return "default-deny";
			}
			catch (e) { }
			try {
				if(this.prefBranch.getBoolPref("enableForOriginatingWebsiteOnly")) {
					behavior = 1;
				}
			}
			catch (e) {

				// hopefully there will be no more changes to the cookie pref
				try {
					behavior = this.prefBranch.getIntPref("cookieBehavior");
				}
				catch (e) {
					dump("can't read network.cookie.cookieBehavior");
				}
			}

			// cookies disabled
			if(behavior == 2) return "default-deny";

			// warn about cookies
			try {
				if(this.prefBranch.getBoolPref("warnAboutCookies")) {
					return "default-warn";
				}
			}
			catch (e) {
				dump("can't read network.cookie.warnAboutCookies");
			}

			switch (behavior) {
			case 0: // enable third party cookies
				return "default-third-party";
			case 1: // enable cookies
				return "default-allow";
			case 3: // base on p3p
				return "default-p3p";
			}
			// should never happen
			return "default-unknown";
		},

		// check permission
		getPermission : function(location) {
			var permMan = this.permissionManager;
			switch(permMan.testPermission(location,"cookie")) {
				case this.ALLOW_SESSION:
				return "allow-session";
				case permMan.ALLOW_ACTION:
				return "allow";
				case permMan.DENY_ACTION:
				return "deny";
				default:
				return this.getDefaultPref();
			}
		},

		// set permission
		setPermission : function (option) {
			var location = this.browser.currentURI;
			var permMan = this.permissionManager;
			permMan.remove(location.host,"cookie");
			switch(option.getAttribute("value")) {
			case "allow-session":
				permMan.add(location,"cookie",this.ALLOW_SESSION);
			break;
			case "allow":
				permMan.add(location,"cookie",permMan.ALLOW_ACTION);
			break;
			case "deny":
				permMan.add(location,"cookie",permMan.DENY_ACTION);
			break;
			}
			this.update();
		},

		// generate label from value and location
		labelGenerator : function (type,location) {
			if(type.substr(0,7)=="default") {
				switch(type) {
				case "default":
					return "Cookie Permission";
				case "default-session":
					return "Default (Accept session cookies)";
				case "default-third-party-session":
					return "Default (Accept session third party cookies)";
				case "default-third-party":
					return "Default (Accept third party cookies)";
				case "default-allow":
					return "Default (Accept cookies)";
				case "default-deny":
					return "Default (Reject cookies)";
				case "default-warn":
					return "Default (Warn before accepting cookie)";
				case "default-p3p":
					return "Default (Based on P3P settings)";
				}
			}
			else {
				switch(type) {
				case "allow-session":
					return "Accept session cookies from "+location.host;
				case "allow":
					return "Accept cookies from "+location.host;
				case "deny":
					return "Reject cookies from "+location.host;
				}
			}
			return "Cookie Permission";
		},

		// update popup/dropdown menu
		updateMenu : function (popup) {
			// shouldn't need this here, but we seem to be hitting a firebird bug where the webProgressListener sometimes fail to trigger
			if(!this.update()) return false;
			var popupitems = popup.getElementsByTagName("menuitem");
			var location = this.browser.currentURI;

			var value = this.getPermission(location);
			var default_value = (value.substr(0,7)!="default") ? this.getDefaultPref() : value;

			// set poup/dropdown label/values
			for (var i = 0; i < popupitems.length; i++) {
				var option = popupitems[i].getAttribute("value");
				if (option.substr(0,7)=="default") {
					option = default_value;
					popupitems[i].setAttribute("value",option);
				}
				if (option == value) popupitems[i].setAttribute("checked","true");
				popupitems[i].setAttribute("label",this.labelGenerator(option,location));
			}
			return true;
		}
	};
	// initializing
	broadcaster._cookiePermission.init();
},false);
